# ========================================================
# Checker's interface
# ========================================================

class DomainServicesOperabilityCheckerInterface(object):
	"""Interface class"""

	def check_website_availability(self, domains_to_check):
		"""Check whether the index pages of specified domains can be retrieved from the specified IP addresses over HTTP and, optionally, HTTPS.
		   @param domains_to_check	list of items describing what should be checked.
		   @return			list of DomainIssue items describing the problems occurred.
		   For possible issue types check 'WebsiteAvailabilityIssueType'
		"""
		raise NotImplementedError()

	def check_dns(
		self, domains_to_check, 
		max_difference_lines=3, save_external_report_data_function=None, 
		dns_server_name='PPA DNS server',
		difference_file_panel_id='ppa'
	):
		"""Check whether all necessary DNS records of the specified domains are returned by specified DNS servers.
		   @param domains_to_check	list of DomainDNSService items describing what should be checked.
		   @return			list of DomainIssue items describing the problems occurred.
		   For possible issue types check 'DNSIssueType'
		"""
		raise NotImplementedError()

	def check_mail_auth(self, domains_to_check):
		"""Check whether all mail users of the specified domains can authenticate over IMAP and SMTP on the specified mail server.
		   @param domains_to_check	list of DomainMailService items describing what should be checked.
		   @return			list of DomainIssue items describing the problems occurred.
		   For possible issue types check 'MailAuthIssueType'
		"""
		raise NotImplementedError()

	def check_ftp_auth(self, domains_to_check):
		"""Check whether all system users can authenticate over FTP on the specified web server.
		   @param domains_to_check	list of DomainFTPService items describing what should be checked.
		   @return			list of DomainIssue items describing the problems occurred.
		   For possible issue types check 'FTPAuthIssueType'
		"""
		raise NotImplementedError()

# ========================================================
# Data types you have to pass to the checker or get from the checker
# ========================================================

class DomainIssue(object):
	"""This structure describes the items of the returned list of issues."""
	def __init__(self, domain_name, category, severity, problem):
		"""Arguments:
		'domain_name' - [mandatory] Domain name. Type: unicode string.
		'category' - [mandatory] Category of the problem, intended for sorting and ordering the similar problems. Type: string.
		'severity' - [mandatory] Severity of the problem. Type: string.
		'problem' - [mandatory] Problem statement: what is wrong, how it impacts against the goal. Type: string."""
		self.domain_name = domain_name
		self.category = category
		self.severity = severity
		self.problem = problem

	def __repr__(self):
		return "DomainIssue(domain_name=%r, category=%r, severity=%r, problem=%r)" % (
			self.domain_name, self.category, self.severity, self.problem
		)

	def __eq__(self, another):
		return (
			self.domain_name == another.domain_name and
			self.category == another.category and
			self.severity == another.severity and
			self.problem == another.problem
		)

class Severity(object):
	"""Supported severity values."""

	INFO = 'info'
	WARNING = 'warning'
	ERROR = 'error'

class MailAuthIssueType:
	"""Issue types when checking mail auth"""

	# Domain has no mail IP address, so we can not check it
	MAIL_IP_NOT_DEFINED = 'mail_ip_not_defined'
	# Failed to connect, failed to login or any other netword/protocol issue when checking login by IMAP
	CHECK_FAILED_IMAP = 'connection_failed_imap'
	# Unexpected internal error when checking login by IMAP
	INTERNAL_ERROR_IMAP = 'internal_error_imap'
	# Failed to connect, failed to login or any other netword/protocol issue when checking login by SMTP
	CHECK_FAILED_SMTP = 'connection_failed_smtp'
	# Unexpected internal error when checking login by SMTP
	INTERNAL_ERROR_SMTP = 'internal_error_smtp'

class FTPAuthIssueType:
	"""Issue types when checking FTP auth"""

	# Domain has no web IP address, so we can not check it
	WEB_IP_NOT_DEFINED = 'web_ip_not_defined'
	# Failed to connect, failed to login or any other network/protocol issue when checking login by FTP
	CHECK_FAILED = 'check_failed'
	# Unexpected internal error when checking login by FTP
	INTERNAL_ERROR = 'internal_error'

class DNSIssueType:
	"""Issue types when checking DNS"""

	# DNS server returned records that differ from expected
	DIFFERENT_RECORDS = 'different_records'
	# Request to DNS server times out
	DNS_SERVER_TIMEOUT = 'dns_server_timeout'
	# Unexpected internal error when checking DNS records
	INTERNAL_ERROR = 'internal_error'

class WebsiteAvailabilityIssueType:
	"""Issue types when checking web."""

	# Web server returned 500-series code for index page request
	STATUS_CODE_5xx = 'status_code_5xx'
	# Connection issue when connecting to web server
	CONNECTION_ISSUE = 'connection_issue'
	# Unexpected internal error when checking web index page
	INTERNAL_ERROR = 'internal_error'

class WebIssueType:
	# Internal operability checker error
	INTERNAL_ERROR = 'internal_error'
	# Source and target domain web page content differ
	WEBPAGE_CONTENT_DIFFERS = 'webpage_content_differs'
	# HTTP client's request failed 
	REQUEST_FAILED = 'request_failed' 
	# Cannot compare web page content with HTTP response that has no content
	CANNOT_COMPARE_20X_WITH_30X = 'cannot_compare_20x_with_30x'
	# Target server HTTP redirect locations differ from that of a source server
	REDIRECT_LOCATIONS_DIFFER = 'redirect_locations_differ'
	# Known error messages found in server response
	FOUND_KNOWN_ERRORS = 'found_known_errors'
	# There are known error messages in source server response
	FOUND_KNOWN_ERRORS_ON_SOURCE = 'found_known_errors_on_source'
	# There are new known error messages in target server response
	FOUND_NEW_ERRORS_ON_TARGET = 'found_new_errors_on_target'
	# Page content markup differs, but page content may be the same
	WEBPAGE_MARKUP_DIFFERS = 'webpage_markup_differs'

class WebProtocol:
	"""Protocols to check web service on"""

	HTTP = 'http'
	HTTPS = 'https'

class DomainWebService(object):
	def __init__(self, domain_name, web_server_ip, protocol):
		"""Arguments:
		'domain_name' - [mandatory] Domain name.
		'web_server_ip' - [mandatory] IP address of the web server. Type: string.
		'protocol' - [mandatory] protocol to check: 'http' or 'https'. Type: string."""
		self.domain_name = domain_name
		self.web_server_ip = web_server_ip
		self.protocol = protocol

class DNSRecord(object):
	def __init__(self, rec_type, src, dst, opt=''):
		"""Arguments:
		'rec_type' - [mandatory] Record type. Type: string.
		'src' - [mandatory] Record source. Type: string.
		'dst' - [mandatory] Record destination. Type: string.
		'opt' - [optional] Record's opt value (such as MX priority). Specify the opt value or None. Type: string."""
		self.rec_type = rec_type
		self.src = src
		self.dst = dst
		self.opt = opt

class DomainDNSService(object):
	def __init__(self, domain_name, dns_records, dns_servers):
		"""Arguments:
		'domain_name' -	[mandatory] Domain name. Type: unicode string.
		'dns_records' - [mandatory] The DNS records that must be returned in response to query for domain. Type: list of DNSRecord items.
		'dns_servers' - [mandatory] The DNS servers to query about this domain. Type: list of IPv4 address strings."""
		self.domain_name = domain_name
		self.dns_records = dns_records
		self.dns_servers = dns_servers

class User(object):
	def __init__(self, login, password):
		"""Arguments:
		'login' - [mandatory] Login. Do not include the domain name here. Type: string.
		'password' - [mandatory] Password. Type: string."""
		self.login = login
		self.password = password

	def __repr__(self):
		return 'User(login=%r, password=%r)' % (self.login, self.password,)

class DomainMailService(object):
	def __init__(self, domain_name, mail_server_ip, users):
		"""Arguments:
		'domain_name' - [mandatory] Domain name. Type: unicode string.
		'mail_server_ip' - [mandatory] IP address of the mail server. Type: string.
		'users' - [mandatory] Users to check the authentication as. Type: list of User items."""
		self.domain_name = domain_name
		self.mail_server_ip = mail_server_ip
		self.users = users

class DomainFTPService(object):
	def __init__(self, domain_name, web_server_ip, users):
		"""Arguments:
		'domain_name' - [mandatory] Domain name. Type: unicode string.
		'web_server_ip' - [mandatory] IP address of the web server. Type: string.
		'users' - [mandatory] Users to check the authentication as. Type: list of User items."""
		self.domain_name = domain_name
		self.web_server_ip = web_server_ip
		self.users = users

# ========================================================
# Concrete implementation of the checker 
# ========================================================

class DomainServicesOperabilityChecker(DomainServicesOperabilityCheckerInterface):
	"""Implementation class for DomainServicesOperabilityCheckerInterface"""
	def check_website_availability(self, domains_to_check):
		from parallels.operability_check.checkers.website_availability_checker import WebsiteAvailabilityChecker
		checker = WebsiteAvailabilityChecker()
		return checker.check(domains_to_check)

	def check_dns(
		self, domains_to_check, 
		max_difference_lines=3, save_external_report_data_function=None, 
		dns_server_name='PPA DNS server',
		difference_file_panel_id='ppa'
	):
		from parallels.operability_check.checkers.dns_checker import DNSChecker
		checker = DNSChecker()
		return checker.check(
			domains_to_check, 
			max_difference_lines, save_external_report_data_function, 
			dns_server_name=dns_server_name,
			difference_file_panel_id=difference_file_panel_id
		)

	def check_mail_auth(self, domains_to_check):
		from parallels.operability_check.checkers.mail_auth_checker import MailAuthChecker
		checker = MailAuthChecker()
		return checker.check(domains_to_check)

	def check_ftp_auth(self, domains_to_check):
		from parallels.operability_check.checkers.ftp_auth_checker import FTPAuthChecker 
		checker = FTPAuthChecker()
		return checker.check(domains_to_check)

