import logging

import imaplib
import smtplib
import socket

from parallels.operability_check import DomainIssue
from parallels.operability_check import Severity, MailAuthIssueType

logger = logging.getLogger(__name__)

problem_messages = {
	MailAuthIssueType.MAIL_IP_NOT_DEFINED:
		u"The domain does not have a mail service IP address.",
	MailAuthIssueType.CHECK_FAILED_IMAP:
		u"Unable to log in to mailboxes over IMAP:\n%s",
	MailAuthIssueType.INTERNAL_ERROR_IMAP:
		u"Internal errors have occurred while checking if it is possible to log in to mailboxes over IMAP:\n%s",
	MailAuthIssueType.CHECK_FAILED_SMTP:
		u"Unable to log in to mailboxes over SMTP:\n%s",
	MailAuthIssueType.INTERNAL_ERROR_SMTP:
		u"Internal errors have occurred while checking if it is possible to log in to mailboxes over SMTP:\n%s",
}

class MailAuthChecker(object):
	def check(self, domains_to_check):
		issues = []

		for domain_to_check in domains_to_check:
			issues += self._check_single_domain(domain_to_check)
		return issues

	def _check_single_domain(self, domain_to_check):
		issues = []

		def add_issue(severity, category, problem):
			issues.append(DomainIssue(domain_name=domain_to_check.domain_name, severity=severity, category=category, problem=problem))

		if domain_to_check.mail_server_ip is None:
			add_issue(
				severity=Severity.WARNING, category=MailAuthIssueType.MAIL_IP_NOT_DEFINED,
				problem=problem_messages[MailAuthIssueType.MAIL_IP_NOT_DEFINED]
			)
			return issues 

		imap_errors = []
		imap_internal_errors = []
		smtp_errors = []
		smtp_internal_errors = []

		for user in domain_to_check.users:
			try:
				self._check_imap_login(domain_to_check.mail_server_ip, user.login, user.password)
			except (imaplib.IMAP4.error, socket.error), e:
				logger.debug(u"Exception:", exc_info=e)
				imap_errors.append((user.login, str(e),))
			except KeyboardInterrupt:
				# for compatibility with python 2.4
				raise
			except Exception, e:
				logger.debug(u"Exception:", exc_info=e)
				imap_internal_errors.append((user.login, str(e),))

			try:
				self._check_smtp_login(domain_to_check.mail_server_ip, user.login, user.password)
			except (smtplib.SMTPException, socket.error), e:
				logger.debug(u"Exception:", exc_info=e)
				smtp_errors.append((user.login, str(e),))
			except KeyboardInterrupt:
				# for compatibility with python 2.4
				raise
			except Exception, e:
				logger.debug(u"Exception:", exc_info=e)
				smtp_internal_errors.append((user.login, str(e),))

		if len(imap_errors) > 0:
			add_issue(
				severity=Severity.ERROR, category=MailAuthIssueType.CHECK_FAILED_IMAP,
				problem=self._format_error_list(problem_messages[MailAuthIssueType.CHECK_FAILED_IMAP], imap_errors)
			)

		if len(imap_internal_errors) > 0:
			add_issue(
				severity=Severity.WARNING, category=MailAuthIssueType.INTERNAL_ERROR_IMAP,
				problem=self._format_error_list(problem_messages[MailAuthIssueType.INTERNAL_ERROR_IMAP], imap_internal_errors)
			)

		if len(smtp_errors) > 0:
			add_issue(
				severity=Severity.ERROR, category=MailAuthIssueType.CHECK_FAILED_SMTP,
				problem=self._format_error_list(problem_messages[MailAuthIssueType.CHECK_FAILED_SMTP], smtp_errors)
			)

		if len(smtp_internal_errors) > 0:
			add_issue(
				severity=Severity.WARNING, category=MailAuthIssueType.INTERNAL_ERROR_SMTP,
				problem=self._format_error_list(problem_messages[MailAuthIssueType.INTERNAL_ERROR_SMTP], smtp_internal_errors)
			)

		return issues

	@staticmethod
	def _format_error_list(error_message, error_list):
		return error_message % (
			u"\n".join(["- %s: %s" % (mailbox_fullname, error_msg) for mailbox_fullname, error_msg in error_list]) 
		)

	def _check_imap_login(self, host, login, password):
		imap = imaplib.IMAP4(host)
		imap.login(login, password)
		imap.list() # execute some command after login
		imap.logout()

	def _check_smtp_login(self, host, login, password):
		smtp = smtplib.SMTP(host)
		smtp.login(login, password) # just login and quit - that's enough
		smtp.quit()

